/**
 * @file src/index.js
 * @license MIT
 * @copyright 2017-present Foko Inc. All rights reserved.
 */

var getPrototypeOf = Object.getPrototypeOf || function legacyGetPrototypeOf (o) {
  /* eslint no-proto: 0 */
  return o.__proto__
}

/**
 * Creates a dirty JSON signature for quick ETag verification.
 * @param {Object} json stringifiable object
 * @param {Object} [paths] a map of paths to ignore
 * @param {Number} max maximum depth to search
 * @param {Number} curr key path traversed so far
 * @param {Array} seen internally used to manage circular object
 * @returns a dirty signature for the object as a string
 */
function genSig (json, paths, max, curr, seen) {
  curr = curr || ''
  seen = seen || []

  // protect against circular objects
  if (seen.length >= max || seen.indexOf(json) !== -1 || json === undefined) return ''
  seen.push(json)

  var type = typeof json

  // quick exit for primitives
  if (type === 'function' || json === null || Number.isNaN(json)) return 'null'
  if (type === 'string') return `'${json}'`

  // number & boolean
  if (type !== 'object') return json

  // built-ins
  var proto = getPrototypeOf(json)

  if (proto !== null) {
    if ('toLocaleTimeString' in proto) return +json
    if ('test' in proto && 'exec' in proto) return json
  }

  // quick stringify
  var sigg = 'length' in json && 'map' in json ? '[' : '{'
  for (var k in json) {
    var kk = curr ? `${curr}.${k}` : k

    if (!paths || paths[kk] !== false) {
      sigg += k + ':' + genSig(json[k], paths, max, kk, seen)
    }
  }
  return sigg
}

/**
 * Generates an ETag for a given entity.
 * @param {any} body entity for which to generate a tag
 * @param {Object} [ignorePaths] a map of paths to ignore
 * @param {Number} [depth=10] maximum depth to traverse
 * @returns a weak ETag representing the entity
 */
function createEtag (body, ignorePaths, depth) {
  depth = depth || 10

  var sig = String(genSig(body, typeof ignorePaths === 'object' && ignorePaths, depth))
  return `W/"${Buffer.from(sig).toString('base64')}"`
}

module.exports = createEtag
module.exports.ziptag = createEtag
